==> ui-18.0.5.tgz: package/components/dropdown/Sub.js <==
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEventCallback, useRefExtra } from '@react-devui/hooks';
import { RightOutlined } from '@react-devui/icons';
import { checkNodeExist, getClassName, getHorizontalSidePosition } from '@react-devui/utils';
import { TTANSITION_DURING_POPUP, WINDOW_SPACE } from '../../utils';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { usePrefixConfig, useTranslation } from '../root';
function Sub(props, ref) {
    const { children, dId, dLevel, dIcon, dList, dPopupState, dTrigger, dZIndex, dEmpty, dFocusVisible, dDisabled, onVisibleChange } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const liRef = useRef(null);
    const ulRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${dPrefix}dropdown-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${dPrefix}dropdown-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    //#endregion
    const [t] = useTranslation();
    const isVisible = !isUndefined(dPopupState);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (isVisible && ulRef.current && liRef.current) {
            const [width, height] = [ulRef.current.offsetWidth, ulRef.current.offsetHeight];
            const { top, left, transformOrigin } = getHorizontalSidePosition(liRef.current, { width, height }, {
                placement: 'right',
                inWindow: WINDOW_SPACE,
            });
            setPopupPositionStyle({
                top,
                left,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    useImperativeHandle(ref, () => updatePosition, [updatePosition]);
    const liNode = (_jsxs("li", Object.assign({ ref: liRef, id: dId, className: getClassName(`${dPrefix}dropdown__item`, `${dPrefix}dropdown__item--sub`, {
            'is-expand': isVisible,
            'is-disabled': dDisabled,
        }), style: { paddingLeft: 12 + dLevel * 16 }, role: "menuitem", "aria-haspopup": true, "aria-expanded": isVisible, "aria-disabled": dDisabled }, { children: [dFocusVisible && _jsx("div", { className: `${dPrefix}focus-outline` }), checkNodeExist(dIcon) && _jsx("div", Object.assign({ className: `${dPrefix}dropdown__item-icon` }, { children: dIcon })), _jsx("div", Object.assign({ className: `${dPrefix}dropdown__item-content` }, { children: children })), _jsx(RightOutlined, { className: `${dPrefix}dropdown__sub-arrow`, dSize: 14 })] })));
    return dDisabled ? (liNode) : (_jsx(DPopup, Object.assign({ dVisible: dPopupState !== null && dPopupState !== void 0 ? dPopupState : false, dTrigger: dTrigger, dUpdatePosition: {
            fn: updatePosition,
            triggerRef: liRef,
            popupRef: ulRef,
            extraScrollRefs: [],
        }, onVisibleChange: onVisibleChange }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(liNode), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: isVisible, dDuring: TTANSITION_DURING_POPUP, onEnter: updatePosition }, { children: (state) => {
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaved':
                                    transitionStyle = { display: 'none' };
                                    break;
                                default:
                                    break;
                            }
                            return renderPopup(_jsx("ul", Object.assign({ ref: ulRef, className: `${dPrefix}dropdown__sub-popup`, style: Object.assign(Object.assign(Object.assign({}, popupPositionStyle), { zIndex: dZIndex }), transitionStyle), role: "menu" }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}dropdown__empty`, style: { paddingLeft: 12 + dLevel * 16 } }, { children: t('No Data') }))) : (dList) })));
                        } })), containerRef.current)] })) })));
}
export const DSub = React.forwardRef(Sub);

==> ui-18.6.2.tgz: package/components/dropdown/Sub.js <==
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEventCallback, useRefExtra } from '@react-devui/hooks';
import { RightOutlined } from '@react-devui/icons';
import { checkNodeExist, getClassName } from '@react-devui/utils';
import { getHorizontalSidePosition, TTANSITION_DURING_POPUP, WINDOW_SPACE } from '../../utils';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { usePrefixConfig, useTranslation } from '../root';
function Sub(props, ref) {
    const { children, dId, dLevel, dIcon, dList, dPopupState, dTrigger, dZIndex, dEmpty, dFocusVisible, dDisabled, onVisibleChange } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const liRef = useRef(null);
    const ulRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${dPrefix}dropdown-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${dPrefix}dropdown-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    //#endregion
    const [t] = useTranslation();
    const isVisible = !isUndefined(dPopupState);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (isVisible && ulRef.current && liRef.current) {
            const [width, height] = [ulRef.current.offsetWidth, ulRef.current.offsetHeight];
            const { top, left, transformOrigin } = getHorizontalSidePosition(liRef.current, { width, height }, {
                placement: 'right',
                inWindow: WINDOW_SPACE,
            });
            setPopupPositionStyle({
                top,
                left,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    useImperativeHandle(ref, () => updatePosition, [updatePosition]);
    const liNode = (_jsxs("li", Object.assign({ ref: liRef, id: dId, className: getClassName(`${dPrefix}dropdown__item`, `${dPrefix}dropdown__item--sub`, {
            'is-expand': isVisible,
            'is-disabled': dDisabled,
        }), style: { paddingLeft: 12 + dLevel * 16 }, role: "menuitem", "aria-haspopup": true, "aria-expanded": isVisible, "aria-disabled": dDisabled }, { children: [dFocusVisible && _jsx("div", { className: `${dPrefix}focus-outline` }), checkNodeExist(dIcon) && _jsx("div", Object.assign({ className: `${dPrefix}dropdown__item-icon` }, { children: dIcon })), _jsx("div", Object.assign({ className: `${dPrefix}dropdown__item-content` }, { children: children })), _jsx(RightOutlined, { className: `${dPrefix}dropdown__sub-arrow`, dSize: 14 })] })));
    return dDisabled ? (liNode) : (_jsx(DPopup, Object.assign({ dVisible: dPopupState !== null && dPopupState !== void 0 ? dPopupState : false, dTrigger: dTrigger, dUpdatePosition: {
            fn: updatePosition,
            triggerRef: liRef,
            popupRef: ulRef,
            extraScrollRefs: [],
        }, onVisibleChange: onVisibleChange }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(liNode), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: isVisible, dDuring: TTANSITION_DURING_POPUP, onEnter: updatePosition }, { children: (state) => {
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaved':
                                    transitionStyle = { display: 'none' };
                                    break;
                                default:
                                    break;
                            }
                            return renderPopup(_jsx("ul", Object.assign({ ref: ulRef, className: `${dPrefix}dropdown__sub-popup`, style: Object.assign(Object.assign(Object.assign({}, popupPositionStyle), { zIndex: dZIndex }), transitionStyle), role: "menu" }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}dropdown__empty`, style: { paddingLeft: 12 + dLevel * 16 } }, { children: t('No Data') }))) : (dList) })));
                        } })), containerRef.current)] })) })));
}
export const DSub = React.forwardRef(Sub);
